#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Données anonymisées
# 
# Ces données ont été générées par une quarantaine de personnes et sont mises à disposition une fois anonymisées.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## Observations brutes

# In[3]:


import pandas
df = pandas.read_csv("logs/qcm100.txt", encoding="utf-8")
df.head()


# ## Préparation des données

# In[4]:


df2 = df[df.qtime == 'end'].copy()
df2.head().T.head()


# ## Premier jeu : simple_french_qcm

# In[5]:


person_id = ['person_id']


# In[6]:


cols = person_id + [c for c in df2.columns if "simple_french_qcm" in c and '-a' in c]
df_question = df2[cols]
cols = person_id + [c for c in df2.columns if "simple_french_qcm" in c and '-b' in c]
df_bouton = df2[cols]
cols = person_id + [c for c in df2.columns if "simple_french_qcm" in c and '-nb' in c]
df_visit = df2[cols]
cols = person_id + [c for c in df2.columns if "simple_french_qcm" in c and '-ANS' in c]
df_ans = df2[cols]
cols = person_id + [c for c in df2.columns if "simple_french_qcm" in c and '-dur' in c]
df_dur = df2[cols]


# In[7]:


df_question.shape


# In[8]:


df_dur.head().T


# In[9]:


import numpy 

def aggnotnan_serie(values):
    res = []
    for v in values:
        if isinstance(v, float) and numpy.isnan(v):
            continue
        if pandas.isnull(v):
            continue
        if v in ('ok', 'on'):
            v = 1
        elif v == 'skip':
            v = 1000
        res.append(v)
    if len(res) > 0:        
        if isinstance(res[0], str):
            r = ",".join(str(_) for _ in res)
        else:
            if len(res) == 1:
                r = res[0]
            else:
                try:
                    r = sum(res)
                except:
                    r = 0
    else:
        r = numpy.nan
    return r


def aggnotnan(values):
    if isinstance(values, pandas.core.series.Series):
        r = aggnotnan_serie(values)
        return r
    else:
        res = []
        for col in values.columns:
            val = list(values[col])
            res.append(aggnotnan_serie(val))
        df = pandas.DataFrame(res, columns)
        return df
    
gr_ans = df_ans.groupby("person_id").agg(aggnotnan)
gr_ans


# In[10]:


gr_dur = df_dur.groupby("person_id").agg(aggnotnan)
gr_dur.head().T


# In[11]:


gr_question = df_question.groupby("person_id").agg(aggnotnan)
gr_question.head().T


# In[12]:


gr_question.shape


# In[13]:


gr_bouton = df_bouton.groupby("person_id").agg(aggnotnan)
gr_bouton.head().T


# In[14]:


gr_visit = df_visit.groupby("person_id").agg(aggnotnan)
gr_visit.head().T


# ### Histogrammes

# In[15]:


nonan_question = gr_question.fillna(0)


# In[16]:


nonan_question.shape


# In[17]:


import matplotlib.pyplot as plt

fig, ax = plt.subplots(nonan_question.shape[0], 1, 
                       figsize=(8, nonan_question.shape[0]))
for i in range(0, nonan_question.shape[0]):
    ax[i].set_ylabel(nonan_question.index[i])
    ax[i].bar(list(range(nonan_question.shape[1])), 
              nonan_question.iloc[i,:])


# ### ACP

# In[18]:


nonan_question.shape


# In[19]:


from sklearn.decomposition import PCA
acp = PCA(n_components=2, svd_solver="arpack")
acp.fit(nonan_question)


# In[20]:


coord = acp.transform(nonan_question)
data = pandas.DataFrame(data=coord, columns=['X1', 'X2'], index=nonan_question.index)
data["cluster"] = 0
data.head()


# In[21]:


import matplotlib.pyplot as plt
fig, ax = plt.subplots(1, 1)
colors = ['red', 'blue', 'orange', 'green']
for i in range(0, 1):
    data[data.cluster==i].plot(x="X1", y="X2", 
                               kind="scatter", 
                               ax=ax, label="c%d" % i, 
                               color=colors[i])


# In[22]:


data["weight"] = 10
# Plot miles per gallon against horsepower with other semantics
fig, ax = plt.subplots(1, 1, figsize=(14, 4))
ax.plot(data.loc[data.cluster == 0, 'X1'].values,
        data.loc[data.cluster == 0, 'X2'].values, "ro", label="c0")
ax.plot(data.loc[data.cluster == 1, 'X1'].values,
        data.loc[data.cluster == 1, 'X2'].values, "bo", label="c1")
ind = list(data.index)
for i in range(0, data.shape[0]):
    ax.text(data.iloc[i, 0], data.iloc[i, 1], ind[i])
ax.legend();


# ## Second jeu de données

# In[23]:


df2 = df[df.qtime == 'end'].copy()
person_id = ['person_id']


# In[24]:


cols = person_id + [c for c in df2.columns if "ml_french" in c and '-a' in c]
df2_question = df2[cols]


# In[25]:


import missingno as msno
msno.matrix(df2_question, figsize=(15,3));


# In[26]:


cols = person_id + [c for c in df2.columns if "ml_french" in c and '-b' in c]
df2_bouton = df2[cols]
cols = person_id + [c for c in df2.columns if "ml_french" in c and '-nb' in c]
df2_visit = df2[cols]
cols = person_id + [c for c in df2.columns if "ml_french" in c and '-dur' in c]
df2_dur = df2[cols]


# In[27]:


gr2_question = df2_question.groupby("person_id").agg(aggnotnan)
gr2_question.T


# In[28]:


msno.matrix(gr2_question, figsize=(10,4));


# ### ACP

# In[29]:


gr2_question.shape


# In[30]:


nonan2_question = gr2_question.fillna(0)


# In[31]:


from sklearn.decomposition import PCA
acp = PCA(n_components=2, svd_solver="arpack")
acp.fit(nonan2_question)


# In[32]:


coord = acp.transform(nonan2_question)
data = pandas.DataFrame(data=coord, columns=['X1', 'X2'], index=nonan_question.index)
data["cluster"] = 0
data


# In[33]:


import matplotlib.pyplot as plt
fig, ax = plt.subplots(1, 1)
colors = ['red', 'blue', 'orange', 'green']
for i in range(0, 1):
    data[data.cluster==i].plot(x="X1", y="X2", 
                               kind="scatter", 
                               ax=ax, label="c%d" % i, 
                               color=colors[i])


# In[34]: